from typing import Any, Iterable, Union

from .mailer import Mailer, registry
from .message import EmailMessage


def get_mailer(name: str = "default") -> Mailer:
    return registry.get(name)


async def send_mail(
    to: Union[str, Iterable[str]], message: EmailMessage, mailer: str = "default"
) -> Any:
    message.to = to
    return await registry.get(mailer).send(message)



from typing import Any, Dict, Union

from .config import EmailURL
from .exceptions import NotRegisteredMailer
from .message import EmailMessage
from .transports import BaseTransport, Transports


class Mailer:
    def __init__(self, url_or_transport: Union[str, EmailURL, BaseTransport]) -> None:
        if isinstance(url_or_transport, (str, EmailURL)):
            url_or_transport = Transports.from_url(url_or_transport)

        self._transport = url_or_transport

    async def send(self, message: EmailMessage) -> Any:
        return await self._transport.send(message)


class MailerRegistry:
    def __init__(self) -> None:
        self._mailers: Dict[str, Mailer] = {}

    def add(self, name: str, mailer: Union[str, EmailURL, Mailer]) -> None:
        if isinstance(mailer, (str, EmailURL)):
            mailer = Mailer(mailer)
        self._mailers[name] = mailer

    def get(self, name: str) -> Mailer:
        if name not in self:
            raise NotRegisteredMailer(f'Mailer with name "{name}" not registered.')

        return self._mailers[name]

    def __contains__(self, item: str) -> bool:
        return item in self._mailers


registry = MailerRegistry()



import abc
import datetime
import os
import ssl
import sys
from typing import Any, Dict, List, Mapping, Optional, Type, Union, cast

from mailers.importer import import_from_string

from .config import EmailURL
from .exceptions import (
    DependencyNotFound,
    ImproperlyConfiguredError,
    NotRegisteredTransportError,
)
from .message import EmailMessage

try:
    import aiofiles
except ImportError:  # pragma: nocover
    aiofiles = None

try:
    import aiosmtplib
except ImportError:  # pragma: nocover
    aiosmtplib = None


class BaseTransport(abc.ABC):
    @abc.abstractmethod
    async def send(self, message: EmailMessage) -> None:  # pragma: nocover
        raise NotImplementedError()


class FileTransport(BaseTransport):
    def __init__(self, directory: str):
        if aiofiles is None:
            raise DependencyNotFound(
                '%s requires "aiofiles" library installed.' % self.__class__.__name__
            )

        if directory is None or directory == "":
            raise ImproperlyConfiguredError(
                'Argument "path" of FileTransport cannot be None.'
            )

        self._directory = directory

    async def send(self, message: EmailMessage) -> None:
        file_name = "message_%s.eml" % datetime.datetime.today().isoformat()
        output_file = os.path.join(self._directory, file_name)
        async with aiofiles.open(output_file, "wb") as stream:
            await stream.write(message.as_string().encode("utf8"))

    @classmethod
    def from_url(cls, url: Union[str, EmailURL]) -> "FileTransport":
        url = EmailURL(url)
        return cls(url.path)


class NullTransport(BaseTransport):
    async def send(self, message: EmailMessage) -> None:
        pass

    @classmethod
    def from_url(cls, *args: Any) -> "NullTransport":
        return cls()


class InMemoryTransport(BaseTransport):
    @property
    def mailbox(self) -> List[EmailMessage]:
        return self._storage

    def __init__(self, storage: List[EmailMessage]):
        self._storage = storage

    async def send(self, message: EmailMessage) -> None:
        self._storage.append(message)

    @classmethod
    def from_url(cls, *args: Any) -> "InMemoryTransport":
        mailbox: List[EmailMessage] = []
        return cls(mailbox)


class StreamTransport(BaseTransport):
    def __init__(self, output: Any):
        self._output = output

    async def send(self, message: EmailMessage) -> None:
        self._output.write(str(message))


class ConsoleTransport(StreamTransport):
    def __init__(self) -> None:
        super().__init__(sys.stdout)


class SMTPTransport(BaseTransport):
    def __init__(
        self,
        host: str = "localhost",
        port: int = 25,
        user: Optional[str] = None,
        password: Optional[str] = None,
        use_tls: Optional[bool] = None,
        timeout: int = 10,
        key_file: Optional[str] = None,
        cert_file: Optional[str] = None,
    ):
        if aiosmtplib is None:
            raise DependencyNotFound(
                '%s requires "aiosmtplib" library installed.' % self.__class__.__name__
            )

        self._host = host
        self._user = user
        self._port = port
        self._password = password
        self._use_tls = use_tls
        self._timeout = timeout
        self._key_file = key_file
        self._cert_file = cert_file

    async def send(self, message: EmailMessage) -> None:
        context = ssl.create_default_context()
        client = aiosmtplib.SMTP(
            hostname=self._host,
            port=self._port,
            use_tls=self._use_tls,
            username=self._user,
            password=self._password,
            timeout=self._timeout,
            tls_context=context,
            client_key=self._key_file,
            client_cert=self._cert_file,
        )
        async with client:
            await client.send_message(message.build_message())

    @classmethod
    def from_url(cls, url: Union[str, EmailURL]) -> "SMTPTransport":
        url = EmailURL(url)

        def _cast_to_bool(value: str) -> bool:
            return value.lower() in ["yes", "1", "on", "true"]

        timeout = url.options.get("timeout", None)
        if timeout:
            timeout = int(timeout)

        use_tls = _cast_to_bool(url.options.get("use_tls", ""))
        key_file = url.options.get("key_file", None)
        cert_file = url.options.get("cert_file", None)

        return cls(
            url.hostname or "localhost",
            url.port or 25,
            url.username,
            url.password,
            use_tls=use_tls,
            timeout=timeout,
            key_file=key_file,
            cert_file=cert_file,
        )


class GMailTransport(SMTPTransport):
    def __init__(self, user: Optional[str], password: Optional[str], timeout: int = 10):
        super().__init__(
            host="smtp.gmail.com",
            port=465,
            user=user,
            password=password,
            use_tls=True,
            timeout=timeout,
        )

    @classmethod
    def from_url(cls, url: Union[str, EmailURL]) -> "GMailTransport":
        url = EmailURL(str(url) + "@default")
        return cls(url.username, url.password)


class MailgunTransport(SMTPTransport):
    def __init__(self, user: Optional[str], password: Optional[str], timeout: int = 10):
        super().__init__(
            host="smtp.mailgun.org",
            port=465,
            user=user,
            password=password,
            use_tls=True,
            timeout=timeout,
        )

    @classmethod
    def from_url(cls, url: Union[str, EmailURL]) -> "MailgunTransport":
        url = EmailURL(str(url) + "@default")
        return cls(url.username, url.password)


class Transports:
    mapping: Dict[str, Union[str, Type[BaseTransport]]] = {
        "smtp": "mailers.transports:SMTPTransport",
        "file": "mailers.transports:FileTransport",
        "null": "mailers.transports:NullTransport",
        "memory": "mailers.transports:InMemoryTransport",
        "console": "mailers.transports:ConsoleTransport",
        "gmail": "mailers.transports:GMailTransport",
        "mailgun": "mailers.transports:MailgunTransport",
    }

    @classmethod
    def bind_url(cls, protocol: str, factory: Union[str, Type[BaseTransport]]) -> None:
        cls.mapping[protocol] = factory

    @classmethod
    def bind_urls(cls, urls: Mapping[str, Union[str, Type[BaseTransport]]]) -> None:
        for protocol, factory in urls.items():
            cls.bind_url(protocol, factory)

    @classmethod
    def from_url(cls, url: Union[str, EmailURL]) -> BaseTransport:
        url = EmailURL(url)
        if url.transport not in cls.mapping:
            raise NotRegisteredTransportError(
                f'No transport found with scheme "{url.transport}".'
            )

        klass = cls.mapping[url.transport]
        if isinstance(klass, str):
            klass = import_from_string(klass)

        klass = cast(Type[BaseTransport], klass)
        factory = getattr(klass, "from_url", None)
        if factory is not None:
            return factory(url)

        return klass()



"""Configuration via URL.

Based on https://github.com/encode/databases/blob/master/databases/core.py#L323
"""
import typing
from urllib.parse import SplitResult, parse_qsl, urlsplit


class EmailURL:
    _components: SplitResult

    def __init__(self, url: typing.Union[str, "EmailURL"]):
        self._url = str(url)

    @property
    def components(self) -> SplitResult:
        if not hasattr(self, "_components"):
            self._components: SplitResult = urlsplit(self._url)
        return self._components

    @property
    def transport(self) -> str:
        return self.components.scheme

    @property
    def username(self) -> typing.Optional[str]:
        return self.components.username

    @property
    def password(self) -> typing.Optional[str]:
        return self.components.password

    @property
    def hostname(self) -> typing.Optional[str]:
        return self.components.hostname

    @property
    def port(self) -> typing.Optional[int]:
        return self.components.port

    @property
    def netloc(self) -> str:
        return self.components.netloc

    @property
    def path(self) -> str:
        return self.components.path

    @property
    def options(self) -> typing.Dict:
        if not hasattr(self, "_options"):
            options = dict(parse_qsl(self.components.query))
            self._options = options
        return self._options

    def replace(self, **kwargs: typing.Any) -> "EmailURL":
        if (
            "username" in kwargs
            or "password" in kwargs
            or "hostname" in kwargs
            or "port" in kwargs
        ):
            hostname = kwargs.pop("hostname", self.hostname)
            port = kwargs.pop("port", self.port)
            username = kwargs.pop("username", self.username)
            password = kwargs.pop("password", self.password)

            netloc = hostname
            if port is not None:
                netloc += f":{port}"
            if username is not None:
                userpass = username
                if password is not None:
                    userpass += f":{password}"
                netloc = f"{userpass}@{netloc}"

            kwargs["netloc"] = netloc

        if "transport" in kwargs:
            transport = kwargs.pop("transport", self.transport)
            kwargs["scheme"] = f"{transport}"

        components = self.components._replace(**kwargs)
        return self.__class__(components.geturl())

    def __str__(self) -> str:
        return self._url

    def __repr__(self) -> str:
        url = str(self)
        if self.password:
            url = str(self.replace(password="********"))
        return f"{self.__class__.__name__}({repr(url)})"

    def __eq__(self, other: typing.Any) -> bool:
        return str(self) == str(other)



import importlib
from typing import Any

from .exceptions import ImportFromStringError


def import_from_string(spec: str) -> Any:
    if ":" not in spec:
        raise ImportFromStringError(
            'Import string must be in format: "<module>:<attribute>". '
            f'Given: "{spec}"'
        )

    module_name, _, attribute = spec.rpartition(":")
    try:
        module = importlib.import_module(module_name)
        if not hasattr(module, attribute):
            raise ImportFromStringError(
                f'Attribute "{attribute}" not found in module "{module_name}".'
            )

        return getattr(module, attribute)
    except ImportError:
        raise ImportFromStringError(f'Could not import module: "{module_name}"')



import abc
import datetime
import email
import email.encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import AnyStr, Dict, Iterable, List, Optional, Union

from .exceptions import BadHeaderError


class BaseAttachment(abc.ABC):
    def __init__(
        self,
        mime_type: str = "application/octet-stream",
        charset: str = None,
        disposition: str = "attachment",
        content_id: str = None,
        headers: Dict[str, str] = None,
    ):
        self.mime_type = mime_type
        self.charset = charset
        self.disposition = disposition
        self.content_id = content_id
        self.headers = headers or {}

        if content_id:
            self.headers["Content-ID"] = f"<{content_id}>"
            self.headers["X-Attachment-ID"] = content_id

    @abc.abstractmethod
    def build(self) -> MIMEBase:  # pragma: nocover
        raise NotImplementedError()

    def as_string(self) -> str:
        return self.build().as_string()

    def __str__(self) -> str:
        return self.as_string()


class Attachment(BaseAttachment):
    def __init__(
        self,
        file_name: str,
        contents: Union[str, bytes],
        mime_type: str = "application/octet-stream",
        disposition: str = "attachment",
        charset: str = None,
        content_id: str = None,
        headers: Dict[str, str] = None,
    ):
        self.file_name = file_name
        self.contents = contents
        super().__init__(
            mime_type=mime_type,
            charset=charset,
            disposition=disposition,
            content_id=content_id,
            headers=headers,
        )

    def read(self) -> Union[str, bytes]:
        return self.contents

    def build(self) -> MIMEBase:
        main_type, subtype = self.mime_type.split("/")
        part = MIMEBase(main_type, subtype)
        part.add_header(
            "Content-Disposition", self.disposition, filename=self.file_name
        )
        part.set_payload(self.read(), self.charset)
        for header_name, header_value in self.headers.items():
            part.add_header(header_name, header_value)
        return part


def _ensure_list(value: Union[str, Iterable[str]]) -> List[str]:
    if value is None:
        return []

    if isinstance(value, str):
        return [value]
    return list(value)


def _create_address(address: str, name: str = None) -> str:
    if name:
        return f"{name} <{address}>"
    return address


def _forbid_new_lines(value: str) -> str:
    if value is not None:
        if "\n" in value or "\r" in value:
            raise BadHeaderError(
                f'Header value "{value}" contains new line characters.'
            )
    return value


class EmailMessage:
    _to: List[str]
    _cc: List[str]
    _bcc: List[str]
    _reply_to: List[str]

    def __init__(
        self,
        to: Union[str, List[str]] = None,
        subject: Optional[str] = None,
        text_body: Optional[str] = None,
        from_address: Optional[str] = None,
        cc: Union[str, List[str]] = None,
        bcc: Union[str, List[str]] = None,
        reply_to: Union[str, List[str]] = None,
        html_body: Optional[str] = None,
        attachments: List[Attachment] = None,
        headers: Dict[str, str] = None,
        date: Optional[datetime.datetime] = None,
        boundary: Optional[str] = None,
        charset: Optional[str] = None,
        parts: List[MIMEBase] = None,
        encoding: str = "quoted-printable",
    ):
        self.to = to
        self.cc = cc
        self.bcc = bcc
        self.reply_to = reply_to

        self.subject = subject
        self.from_address = from_address
        self.text_body = text_body
        self.html_body = html_body
        self.attachments = attachments or []
        self.boundary = boundary
        self.charset = charset
        self.headers = headers or {}
        self.parts = parts or []
        self.encoding = encoding
        self.date = date or datetime.datetime.today()

    @property
    def to(self) -> List[str]:
        return self._to

    @to.setter
    def to(self, value: Union[str, Iterable[str]]) -> None:
        self._to = _ensure_list(value)

    @property
    def cc(self) -> List[str]:
        return self._cc

    @cc.setter
    def cc(self, value: Union[str, Iterable[str]]) -> None:
        self._cc = _ensure_list(value)

    @property
    def bcc(self) -> List[str]:
        return self._bcc

    @bcc.setter
    def bcc(self, value: Union[str, Iterable[str]]) -> None:
        self._bcc = _ensure_list(value)

    @property
    def reply_to(self) -> List[str]:
        return self._reply_to

    @reply_to.setter
    def reply_to(self, value: Union[str, Iterable[str]]) -> None:
        self._reply_to = _ensure_list(value)

    @property
    def date(self) -> str:
        return self.headers.get("Date", "")

    @date.setter
    def date(self, date: Union[str, datetime.datetime]) -> None:
        if isinstance(date, datetime.datetime):
            date = date.isoformat()
        self.headers["Date"] = date

    def add_to(self, address: str, name: Optional[str] = None) -> "EmailMessage":
        self._to.append(_create_address(address, name))
        return self

    def add_cc(self, address: str, name: Optional[str] = None) -> "EmailMessage":
        self._cc.append(_create_address(address, name))
        return self

    def add_bcc(self, address: str, name: Optional[str] = None) -> "EmailMessage":
        self._bcc.append(_create_address(address, name))
        return self

    def add_reply_to(self, address: str, name: Optional[str] = None) -> "EmailMessage":
        self._reply_to.append(_create_address(address, name))
        return self

    def add_part(self, part: MIMEBase) -> "EmailMessage":
        self.parts.append(part)
        return self

    def add_attachment(self, attachment: Attachment) -> "EmailMessage":
        self.attachments.append(attachment)
        return self

    def attach(
        self,
        contents: AnyStr,
        file_name: str,
        mime_type: str = "application/octet-stream",
        charset: str = None,
        content_id: str = None,
        headers: Dict[str, str] = None,
        disposition: str = "attachment",
    ) -> "EmailMessage":
        self.add_attachment(
            Attachment(
                file_name=file_name,
                contents=contents,
                mime_type=mime_type,
                charset=charset,
                content_id=content_id,
                headers=headers,
                disposition=disposition,
            )
        )
        return self

    def build_message(self) -> MIMEMultipart:
        envelope = MIMEMultipart(boundary=self.boundary)

        envelope.preamble = "This is a multi-part message in MIME format."

        if self.subject is not None:
            envelope.add_header("Subject", self.subject)

        # RFC 822
        if self.from_address is None:
            raise BadHeaderError('"from_address" attribute was not set.')

        # RFC 822
        if not len(self.to) and not len(self.bcc):
            raise BadHeaderError('Neither "to" or "bcc" attribute was not set.')

        envelope.add_header("From", _forbid_new_lines(self.from_address))
        envelope.add_header("To", _forbid_new_lines(", ".join(self._to)))
        envelope.add_header("Content-Transfer-Encoding", self.encoding)

        if len(self._cc):
            envelope.add_header("Cc", ", ".join(self._cc))

        if len(self._bcc):
            envelope.add_header("Bcc", ", ".join(self._bcc))

        if len(self._reply_to):
            envelope.add_header("Reply-to", ", ".join(self._reply_to))

        for name, value in self.headers.items():
            envelope.add_header(name, value)

        # create content parts
        main_message = MIMEMultipart("alternative")
        envelope.attach(main_message)

        if self.text_body:
            text_part = MIMEText(self.text_body, "plain", self.charset)
            main_message.attach(text_part)

        if self.html_body:
            html_part = MIMEText(self.html_body, "html", self.charset)
            main_message.attach(html_part)

        # add custom parts and attachments at the end to prevent their content
        # be used as preview message in GMail.
        for part in self.parts:
            envelope.attach(part)

        for attachment in self.attachments:
            part = attachment.build()
            email.encoders.encode_base64(part)
            envelope.attach(part)

        return envelope

    def as_string(self) -> str:
        return self.build_message().as_string()

    def __str__(self) -> str:
        return self.as_string()



class MailersError(Exception):
    pass


class SMTPError(MailersError):
    pass


class DependencyNotFound(MailersError):
    pass


class ImproperlyConfiguredError(MailersError):
    pass


class ImportFromStringError(MailersError):
    pass


class NotRegisteredTransportError(MailersError):
    pass


class BadHeaderError(MailersError):
    pass


class NotRegisteredMailer(MailersError):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mailers']

package_data = \
{'': ['*']}

install_requires = \
['aiosmtplib>=1.1,<2.0']

extras_require = \
{'aiofiles': ['aiofiles>=0.4.0,<0.5.0'], 'full': ['aiofiles>=0.4.0,<0.5.0']}

setup_kwargs = {
    'name': 'mailers',
    'version': '0.0.2',
    'description': 'Email delivery for asyncio.',
    'long_description': '<p align="center">\n<a href="https://travis-ci.org/alex-oleshkevich/mailers">\n    <img src="https://api.travis-ci.com/alex-oleshkevich/mailers.svg?branch=master" alt="Build Status">\n</a>\n<a href="https://codecov.io/gh/alex-oleshkevich/mailers">\n    <img src="https://codecov.io/gh/alex-oleshkevich/mailers/branch/master/graph/badge.svg" alt="Coverage">\n</a>\n<a href="https://pypi.org/project/mailers/">\n    <img src="https://badge.fury.io/py/mailers.svg" alt="Package version">\n</a>\n</p>\n\n---\n\n# Mailers for asyncio\n\n## Installation\n\n```bash\npip install mailers\n```\n\nIf you plan to use `FileTransport` you would also need to install \n[`aiofiles`](https://github.com/Tinche/aiofiles) extra:\n\n```bash\npip install mailers[aiofiles]\n```\n\nOr install all optional dependencies at once:\n\n```bash\npip install mailers[full]\n```\n\n\n## Usage\n\nThe package uses two main concepts: mailers and transports. \nThe mailer is a class which abstracts you from the underlying transport\nand the transport does the actual message delivery.\n\n```python\nfrom mailers import EmailMessage, configure, send_mail\n\nconfigure(mailers={\n    \'default\': \'smtp://user:password@localhost:25?timeout=2\'\n})\n\nmessage = EmailMessage(\n    to=\'user@localhost\', from_address=\'from@localhost\',\n    subject=\'Hello\', text_body=\'World!\'\n)\nawait send_mail(\'user@localhost\', message)\n```\n\nOr if you prefer more control on what is going one, take this more verbose path:\n\n```python\nfrom mailers import Mailer, SMTPTransport, EmailMessage\n\nmailer = Mailer(SMTPTransport(\'localhost\', 25))\n\nmessage = EmailMessage(\n    to=\'user@localhost\', from_address=\'from@localhost\',\n    subject=\'Hello\', text_body=\'World!\'\n)\nawait mailer.send(message)\n``` \n\n### Shortcuts\n\nThe packages exports some utility methods to simplify the work: `configure` and `send_mail`.\n\n`configure` methods allows to configure multiple mailers and bind URL specs at once.\n\n```python\nfrom mailers import configure\n\nconfigure(\n    mailers={\n        \'default\': \'smtp://localhost\',\n        \'admin\': \'smtp://localhost\'   \n    },\n    transports={\n        \'myproto\': \'myproto.ImplementationClass\'            \n    }\n)\n``` \n\nIn case you have multiple mailers, the `send_mail` shortcut accepts `mailer` argument to specify the mailer to use:\n```python\nsend_mail(to, message, mailer=\'admin\')\n```\n\nWhen you dont\'t set `mailer` argument, the function will use mailer with name `default`.  \n\n\n## Compose messages\n\nThe arguments and methods of `EmailMessage ` class are self-explanatory so here is some basic example:\n\n```python\nfrom mailers import EmailMessage, Attachment\n\nmessage = EmailMessage(\n    to=\'user@localhost\', \n    from_address=\'from@example.tld\', \n    cc=\'cc@example.com\', \n    bcc=[\'bcc@example.com\'], \n    text_body=\'Hello world!\',\n    html_body=\'<b>Hello world!</b>\',\n    attachments=[\n        Attachment(\'CONTENTS\', \'file.txt\', \'text/plain\'),    \n    ]   \n)\n\n# attachments can be added on demand:\n\nwith open(\'file.txt\', \'r\') as f:\n    message.attach(f.read(), f.name, \'text/plain\')\n\n    # alternatively\n    message.add_attachment(\n        Attachment(f.read(), f.name, \'text/plain\')\n    )\n```\n\n`cc`, `bcc`, `to`, `reply_to` can be either strings or lists of strings.\n\n### A note about attachments\n\nAccessing files is a blocking operation. You may want to use `aiofiles` or alternate library\nwhich reads files in non-blocking mode.\n\nThis package does not implement direct access to files at moment. \nThis is something to do at later stage. \n\n## Transports\n\n### Preinstalled transports\n\nAll transport classes can be found in `mailers.transports` module.\n\n| Class             | Example URL                                       | Description                                                       |\n|-------------------|---------------------------------------------------|-------------------------------------------------------------------|\n| SMTPTransport     | smtp://user:pass@hostname:port?timeout=&use_tls=1 | Sends mails using SMTP protocol.                                  |\n| InMemoryTransport | not available                                     | Stores sent messages in the local variable. See an example below. |\n| FileTransport     | file:///path/to/directory                         | Writes sent messages into directory.                              |\n| NullTransport     | null://                                           | Does not perform any sending.                                     |\n| StreamTransport   | not available                                     | Writes message to an open stream. See an example below.           |\n| ConsoleTransport  | console://                                        | Prints messages into stdout.                                      |\n| GMailTransport    | gmail://username:password                         | Sends via GMail.                                                  |\n| MailgunTransport  | mailgun://username:password                       | Sends via Mailgun.                                                |\n\n\n### Special notes\n\n#### InMemoryTransport\n\n`InMemoryTransport` takes a list and writes outgoing mail into it. \nRead this list to inspect the outbox.\n\n```python\nfrom mailers import InMemoryTransport, EmailMessage\n\nmessage = EmailMessage()\nmailbox = []\ntransport = InMemoryTransport(mailbox)\nawait transport.send(message)\n\nassert message in mailbox\n```\n\n#### StreamTransport\n\nWrites messages into the open stream.\n\n```python\nfrom mailers import StreamTransport, EmailMessage\nfrom io import TextIO\n\nmessage = EmailMessage()\n\ntransport = StreamTransport(output=TextIO())\nawait transport.send(message)\n```\n\n`output` is any IO compatible object.\n\n\n### Custom transports.\n\nEach transport must implement `async def send(self, message: EmailMessage) -> None` method. \nPreferably, inherit from `BaseTransport` class:\n\n```python\nfrom mailers import BaseTransport, Mailer, EmailMessage\n\nclass PrintTransport(BaseTransport):\n    async def send(self, message: EmailMessage) -> None\n        print(str(message))\n\nmailer = Mailer(PrintTransport())\n```\n\nIn order to make your transport to accept `EmailURL` instances, your transport class has to implement `from_url` \nclass method:\n\n```python\nfrom mailers import BaseTransport, EmailMessage, EmailURL\n\nclass PrintTransport(BaseTransport):\n    @classmethod\n    def from_url(cls, url: EmailURL) -> "PrintTransport":\n        return cls()\n```\n\n### Add custom transport protocols.\n\nOnce you build a custom transport you can add it\'s URL to enable URL-based configurations.\n```python\nfrom mailers import Transports, Mailer\n\nTransports.bind_url(\'myprotocol\', \'my.transport.Name\')\n\nmailer = Mailer(\'myprotocol://\')\n``` \n\nNote that the transport must to implement `from_url` method to accept URL parameters.\nOtherwise it will be constructed without any arguments passed to the `__init__` method.\n',
    'author': 'alex.oleshkevich',
    'author_email': 'alex.oleshkevich@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/alex-oleshkevich/mailers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Mapping, Optional, Union

from .config import EmailURL
from .exceptions import MailersError
from .mailer import Mailer, MailerRegistry, registry
from .message import Attachment, EmailMessage
from .shortcuts import get_mailer, send_mail
from .transports import (
    BaseTransport,
    FileTransport,
    GMailTransport,
    InMemoryTransport,
    MailgunTransport,
    NullTransport,
    SMTPTransport,
    StreamTransport,
    Transports,
)

__all__ = [
    "Transports",
    "InMemoryTransport",
    "SMTPTransport",
    "NullTransport",
    "FileTransport",
    "BaseTransport",
    "StreamTransport",
    "GMailTransport",
    "MailgunTransport",
    "MailersError",
    "EmailMessage",
    "Attachment",
    "EmailURL",
    "Mailer",
    "MailerRegistry",
    "registry",
    "send_mail",
    "get_mailer",
]

__version__ = "0.0.2"


def configure(
    mailers: Mapping[str, Union[str, EmailURL, Mailer]],
    transports: Optional[Mapping[str, str]] = None,
) -> None:
    for name, mailer in mailers.items():
        registry.add(name, mailer)

    if transports is not None:
        Transports.bind_urls(transports)
